# This is a generated file, use data-scripts/generatePyUniData.py to edit and regenerate.

nameToUnicode = {
    "A": 0x0041,  # LATIN CAPITAL LETTER A
    "AE": 0x00C6,  # LATIN CAPITAL LETTER AE
    "AEacute": 0x01FC,  # LATIN CAPITAL LETTER AE WITH ACUTE
    "Aacute": 0x00C1,  # LATIN CAPITAL LETTER A WITH ACUTE
    "Abreve": 0x0102,  # LATIN CAPITAL LETTER A WITH BREVE
    "Acircumflex": 0x00C2,  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
    "Adieresis": 0x00C4,  # LATIN CAPITAL LETTER A WITH DIAERESIS
    "Agrave": 0x00C0,  # LATIN CAPITAL LETTER A WITH GRAVE
    "Alpha": 0x0391,  # GREEK CAPITAL LETTER ALPHA
    "Alphatonos": 0x0386,  # GREEK CAPITAL LETTER ALPHA WITH TONOS
    "Amacron": 0x0100,  # LATIN CAPITAL LETTER A WITH MACRON
    "Aogonek": 0x0104,  # LATIN CAPITAL LETTER A WITH OGONEK
    "Aring": 0x00C5,  # LATIN CAPITAL LETTER A WITH RING ABOVE
    "Aringacute": 0x01FA,  # LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
    "Atilde": 0x00C3,  # LATIN CAPITAL LETTER A WITH TILDE
    "B": 0x0042,  # LATIN CAPITAL LETTER B
    "Beta": 0x0392,  # GREEK CAPITAL LETTER BETA
    "C": 0x0043,  # LATIN CAPITAL LETTER C
    "Cacute": 0x0106,  # LATIN CAPITAL LETTER C WITH ACUTE
    "Ccaron": 0x010C,  # LATIN CAPITAL LETTER C WITH CARON
    "Ccedilla": 0x00C7,  # LATIN CAPITAL LETTER C WITH CEDILLA
    "Ccircumflex": 0x0108,  # LATIN CAPITAL LETTER C WITH CIRCUMFLEX
    "Cdotaccent": 0x010A,  # LATIN CAPITAL LETTER C WITH DOT ABOVE
    "Chi": 0x03A7,  # GREEK CAPITAL LETTER CHI
    "D": 0x0044,  # LATIN CAPITAL LETTER D
    "Dcaron": 0x010E,  # LATIN CAPITAL LETTER D WITH CARON
    "Dcroat": 0x0110,  # LATIN CAPITAL LETTER D WITH STROKE
    "Delta": 0x2206,  # INCREMENT
    "E": 0x0045,  # LATIN CAPITAL LETTER E
    "Eacute": 0x00C9,  # LATIN CAPITAL LETTER E WITH ACUTE
    "Ebreve": 0x0114,  # LATIN CAPITAL LETTER E WITH BREVE
    "Ecaron": 0x011A,  # LATIN CAPITAL LETTER E WITH CARON
    "Ecircumflex": 0x00CA,  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
    "Edieresis": 0x00CB,  # LATIN CAPITAL LETTER E WITH DIAERESIS
    "Edotaccent": 0x0116,  # LATIN CAPITAL LETTER E WITH DOT ABOVE
    "Egrave": 0x00C8,  # LATIN CAPITAL LETTER E WITH GRAVE
    "Emacron": 0x0112,  # LATIN CAPITAL LETTER E WITH MACRON
    "Eng": 0x014A,  # LATIN CAPITAL LETTER ENG
    "Eogonek": 0x0118,  # LATIN CAPITAL LETTER E WITH OGONEK
    "Epsilon": 0x0395,  # GREEK CAPITAL LETTER EPSILON
    "Epsilontonos": 0x0388,  # GREEK CAPITAL LETTER EPSILON WITH TONOS
    "Eta": 0x0397,  # GREEK CAPITAL LETTER ETA
    "Etatonos": 0x0389,  # GREEK CAPITAL LETTER ETA WITH TONOS
    "Eth": 0x00D0,  # LATIN CAPITAL LETTER ETH
    "Euro": 0x20AC,  # EURO SIGN
    "F": 0x0046,  # LATIN CAPITAL LETTER F
    "G": 0x0047,  # LATIN CAPITAL LETTER G
    "Gamma": 0x0393,  # GREEK CAPITAL LETTER GAMMA
    "Gbreve": 0x011E,  # LATIN CAPITAL LETTER G WITH BREVE
    "Gcaron": 0x01E6,  # LATIN CAPITAL LETTER G WITH CARON
    "Gcircumflex": 0x011C,  # LATIN CAPITAL LETTER G WITH CIRCUMFLEX
    "Gdotaccent": 0x0120,  # LATIN CAPITAL LETTER G WITH DOT ABOVE
    "H": 0x0048,  # LATIN CAPITAL LETTER H
    "H18533": 0x25CF,  # BLACK CIRCLE
    "H18543": 0x25AA,  # BLACK SMALL SQUARE
    "H18551": 0x25AB,  # WHITE SMALL SQUARE
    "H22073": 0x25A1,  # WHITE SQUARE
    "Hbar": 0x0126,  # LATIN CAPITAL LETTER H WITH STROKE
    "Hcircumflex": 0x0124,  # LATIN CAPITAL LETTER H WITH CIRCUMFLEX
    "I": 0x0049,  # LATIN CAPITAL LETTER I
    "IJ": 0x0132,  # LATIN CAPITAL LIGATURE IJ
    "Iacute": 0x00CD,  # LATIN CAPITAL LETTER I WITH ACUTE
    "Ibreve": 0x012C,  # LATIN CAPITAL LETTER I WITH BREVE
    "Icircumflex": 0x00CE,  # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
    "Idieresis": 0x00CF,  # LATIN CAPITAL LETTER I WITH DIAERESIS
    "Idotaccent": 0x0130,  # LATIN CAPITAL LETTER I WITH DOT ABOVE
    "Ifraktur": 0x2111,  # BLACK-LETTER CAPITAL I
    "Igrave": 0x00CC,  # LATIN CAPITAL LETTER I WITH GRAVE
    "Imacron": 0x012A,  # LATIN CAPITAL LETTER I WITH MACRON
    "Iogonek": 0x012E,  # LATIN CAPITAL LETTER I WITH OGONEK
    "Iota": 0x0399,  # GREEK CAPITAL LETTER IOTA
    "Iotadieresis": 0x03AA,  # GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
    "Iotatonos": 0x038A,  # GREEK CAPITAL LETTER IOTA WITH TONOS
    "Itilde": 0x0128,  # LATIN CAPITAL LETTER I WITH TILDE
    "J": 0x004A,  # LATIN CAPITAL LETTER J
    "Jcircumflex": 0x0134,  # LATIN CAPITAL LETTER J WITH CIRCUMFLEX
    "K": 0x004B,  # LATIN CAPITAL LETTER K
    "Kappa": 0x039A,  # GREEK CAPITAL LETTER KAPPA
    "L": 0x004C,  # LATIN CAPITAL LETTER L
    "Lacute": 0x0139,  # LATIN CAPITAL LETTER L WITH ACUTE
    "Lambda": 0x039B,  # GREEK CAPITAL LETTER LAMDA
    "Lcaron": 0x013D,  # LATIN CAPITAL LETTER L WITH CARON
    "Ldot": 0x013F,  # LATIN CAPITAL LETTER L WITH MIDDLE DOT
    "Lslash": 0x0141,  # LATIN CAPITAL LETTER L WITH STROKE
    "M": 0x004D,  # LATIN CAPITAL LETTER M
    "Mu": 0x039C,  # GREEK CAPITAL LETTER MU
    "N": 0x004E,  # LATIN CAPITAL LETTER N
    "Nacute": 0x0143,  # LATIN CAPITAL LETTER N WITH ACUTE
    "Ncaron": 0x0147,  # LATIN CAPITAL LETTER N WITH CARON
    "Ntilde": 0x00D1,  # LATIN CAPITAL LETTER N WITH TILDE
    "Nu": 0x039D,  # GREEK CAPITAL LETTER NU
    "O": 0x004F,  # LATIN CAPITAL LETTER O
    "OE": 0x0152,  # LATIN CAPITAL LIGATURE OE
    "Oacute": 0x00D3,  # LATIN CAPITAL LETTER O WITH ACUTE
    "Obreve": 0x014E,  # LATIN CAPITAL LETTER O WITH BREVE
    "Ocircumflex": 0x00D4,  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
    "Odieresis": 0x00D6,  # LATIN CAPITAL LETTER O WITH DIAERESIS
    "Ograve": 0x00D2,  # LATIN CAPITAL LETTER O WITH GRAVE
    "Ohorn": 0x01A0,  # LATIN CAPITAL LETTER O WITH HORN
    "Ohungarumlaut": 0x0150,  # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
    "Omacron": 0x014C,  # LATIN CAPITAL LETTER O WITH MACRON
    "Omega": 0x2126,  # OHM SIGN
    "Omegatonos": 0x038F,  # GREEK CAPITAL LETTER OMEGA WITH TONOS
    "Omicron": 0x039F,  # GREEK CAPITAL LETTER OMICRON
    "Omicrontonos": 0x038C,  # GREEK CAPITAL LETTER OMICRON WITH TONOS
    "Oslash": 0x00D8,  # LATIN CAPITAL LETTER O WITH STROKE
    "Oslashacute": 0x01FE,  # LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
    "Otilde": 0x00D5,  # LATIN CAPITAL LETTER O WITH TILDE
    "P": 0x0050,  # LATIN CAPITAL LETTER P
    "Phi": 0x03A6,  # GREEK CAPITAL LETTER PHI
    "Pi": 0x03A0,  # GREEK CAPITAL LETTER PI
    "Psi": 0x03A8,  # GREEK CAPITAL LETTER PSI
    "Q": 0x0051,  # LATIN CAPITAL LETTER Q
    "R": 0x0052,  # LATIN CAPITAL LETTER R
    "Racute": 0x0154,  # LATIN CAPITAL LETTER R WITH ACUTE
    "Rcaron": 0x0158,  # LATIN CAPITAL LETTER R WITH CARON
    "Rfraktur": 0x211C,  # BLACK-LETTER CAPITAL R
    "Rho": 0x03A1,  # GREEK CAPITAL LETTER RHO
    "S": 0x0053,  # LATIN CAPITAL LETTER S
    "SF010000": 0x250C,  # BOX DRAWINGS LIGHT DOWN AND RIGHT
    "SF020000": 0x2514,  # BOX DRAWINGS LIGHT UP AND RIGHT
    "SF030000": 0x2510,  # BOX DRAWINGS LIGHT DOWN AND LEFT
    "SF040000": 0x2518,  # BOX DRAWINGS LIGHT UP AND LEFT
    "SF050000": 0x253C,  # BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
    "SF060000": 0x252C,  # BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
    "SF070000": 0x2534,  # BOX DRAWINGS LIGHT UP AND HORIZONTAL
    "SF080000": 0x251C,  # BOX DRAWINGS LIGHT VERTICAL AND RIGHT
    "SF090000": 0x2524,  # BOX DRAWINGS LIGHT VERTICAL AND LEFT
    "SF100000": 0x2500,  # BOX DRAWINGS LIGHT HORIZONTAL
    "SF110000": 0x2502,  # BOX DRAWINGS LIGHT VERTICAL
    "SF190000": 0x2561,  # BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
    "SF200000": 0x2562,  # BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE
    "SF210000": 0x2556,  # BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE
    "SF220000": 0x2555,  # BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE
    "SF230000": 0x2563,  # BOX DRAWINGS DOUBLE VERTICAL AND LEFT
    "SF240000": 0x2551,  # BOX DRAWINGS DOUBLE VERTICAL
    "SF250000": 0x2557,  # BOX DRAWINGS DOUBLE DOWN AND LEFT
    "SF260000": 0x255D,  # BOX DRAWINGS DOUBLE UP AND LEFT
    "SF270000": 0x255C,  # BOX DRAWINGS UP DOUBLE AND LEFT SINGLE
    "SF280000": 0x255B,  # BOX DRAWINGS UP SINGLE AND LEFT DOUBLE
    "SF360000": 0x255E,  # BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
    "SF370000": 0x255F,  # BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE
    "SF380000": 0x255A,  # BOX DRAWINGS DOUBLE UP AND RIGHT
    "SF390000": 0x2554,  # BOX DRAWINGS DOUBLE DOWN AND RIGHT
    "SF400000": 0x2569,  # BOX DRAWINGS DOUBLE UP AND HORIZONTAL
    "SF410000": 0x2566,  # BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
    "SF420000": 0x2560,  # BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
    "SF430000": 0x2550,  # BOX DRAWINGS DOUBLE HORIZONTAL
    "SF440000": 0x256C,  # BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
    "SF450000": 0x2567,  # BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE
    "SF460000": 0x2568,  # BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE
    "SF470000": 0x2564,  # BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE
    "SF480000": 0x2565,  # BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE
    "SF490000": 0x2559,  # BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE
    "SF500000": 0x2558,  # BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE
    "SF510000": 0x2552,  # BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE
    "SF520000": 0x2553,  # BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE
    "SF530000": 0x256B,  # BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE
    "SF540000": 0x256A,  # BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE
    "Sacute": 0x015A,  # LATIN CAPITAL LETTER S WITH ACUTE
    "Scaron": 0x0160,  # LATIN CAPITAL LETTER S WITH CARON
    "Scedilla": 0x015E,  # LATIN CAPITAL LETTER S WITH CEDILLA
    "Scircumflex": 0x015C,  # LATIN CAPITAL LETTER S WITH CIRCUMFLEX
    "Sigma": 0x03A3,  # GREEK CAPITAL LETTER SIGMA
    "T": 0x0054,  # LATIN CAPITAL LETTER T
    "Tau": 0x03A4,  # GREEK CAPITAL LETTER TAU
    "Tbar": 0x0166,  # LATIN CAPITAL LETTER T WITH STROKE
    "Tcaron": 0x0164,  # LATIN CAPITAL LETTER T WITH CARON
    "Theta": 0x0398,  # GREEK CAPITAL LETTER THETA
    "Thorn": 0x00DE,  # LATIN CAPITAL LETTER THORN
    "U": 0x0055,  # LATIN CAPITAL LETTER U
    "Uacute": 0x00DA,  # LATIN CAPITAL LETTER U WITH ACUTE
    "Ubreve": 0x016C,  # LATIN CAPITAL LETTER U WITH BREVE
    "Ucircumflex": 0x00DB,  # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
    "Udieresis": 0x00DC,  # LATIN CAPITAL LETTER U WITH DIAERESIS
    "Ugrave": 0x00D9,  # LATIN CAPITAL LETTER U WITH GRAVE
    "Uhorn": 0x01AF,  # LATIN CAPITAL LETTER U WITH HORN
    "Uhungarumlaut": 0x0170,  # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
    "Umacron": 0x016A,  # LATIN CAPITAL LETTER U WITH MACRON
    "Uogonek": 0x0172,  # LATIN CAPITAL LETTER U WITH OGONEK
    "Upsilon": 0x03A5,  # GREEK CAPITAL LETTER UPSILON
    "Upsilon1": 0x03D2,  # GREEK UPSILON WITH HOOK SYMBOL
    "Upsilondieresis": 0x03AB,  # GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
    "Upsilontonos": 0x038E,  # GREEK CAPITAL LETTER UPSILON WITH TONOS
    "Uring": 0x016E,  # LATIN CAPITAL LETTER U WITH RING ABOVE
    "Utilde": 0x0168,  # LATIN CAPITAL LETTER U WITH TILDE
    "V": 0x0056,  # LATIN CAPITAL LETTER V
    "W": 0x0057,  # LATIN CAPITAL LETTER W
    "Wacute": 0x1E82,  # LATIN CAPITAL LETTER W WITH ACUTE
    "Wcircumflex": 0x0174,  # LATIN CAPITAL LETTER W WITH CIRCUMFLEX
    "Wdieresis": 0x1E84,  # LATIN CAPITAL LETTER W WITH DIAERESIS
    "Wgrave": 0x1E80,  # LATIN CAPITAL LETTER W WITH GRAVE
    "X": 0x0058,  # LATIN CAPITAL LETTER X
    "Xi": 0x039E,  # GREEK CAPITAL LETTER XI
    "Y": 0x0059,  # LATIN CAPITAL LETTER Y
    "Yacute": 0x00DD,  # LATIN CAPITAL LETTER Y WITH ACUTE
    "Ycircumflex": 0x0176,  # LATIN CAPITAL LETTER Y WITH CIRCUMFLEX
    "Ydieresis": 0x0178,  # LATIN CAPITAL LETTER Y WITH DIAERESIS
    "Ygrave": 0x1EF2,  # LATIN CAPITAL LETTER Y WITH GRAVE
    "Z": 0x005A,  # LATIN CAPITAL LETTER Z
    "Zacute": 0x0179,  # LATIN CAPITAL LETTER Z WITH ACUTE
    "Zcaron": 0x017D,  # LATIN CAPITAL LETTER Z WITH CARON
    "Zdotaccent": 0x017B,  # LATIN CAPITAL LETTER Z WITH DOT ABOVE
    "Zeta": 0x0396,  # GREEK CAPITAL LETTER ZETA
    "a": 0x0061,  # LATIN SMALL LETTER A
    "aacute": 0x00E1,  # LATIN SMALL LETTER A WITH ACUTE
    "abreve": 0x0103,  # LATIN SMALL LETTER A WITH BREVE
    "acircumflex": 0x00E2,  # LATIN SMALL LETTER A WITH CIRCUMFLEX
    "acute": 0x00B4,  # ACUTE ACCENT
    "acutecomb": 0x0301,  # COMBINING ACUTE ACCENT
    "adieresis": 0x00E4,  # LATIN SMALL LETTER A WITH DIAERESIS
    "ae": 0x00E6,  # LATIN SMALL LETTER AE
    "aeacute": 0x01FD,  # LATIN SMALL LETTER AE WITH ACUTE
    "agrave": 0x00E0,  # LATIN SMALL LETTER A WITH GRAVE
    "aleph": 0x2135,  # ALEF SYMBOL
    "alpha": 0x03B1,  # GREEK SMALL LETTER ALPHA
    "alphatonos": 0x03AC,  # GREEK SMALL LETTER ALPHA WITH TONOS
    "amacron": 0x0101,  # LATIN SMALL LETTER A WITH MACRON
    "ampersand": 0x0026,  # AMPERSAND
    "angle": 0x2220,  # ANGLE
    "angleleft": 0x2329,  # LEFT-POINTING ANGLE BRACKET
    "angleright": 0x232A,  # RIGHT-POINTING ANGLE BRACKET
    "anoteleia": 0x0387,  # GREEK ANO TELEIA
    "aogonek": 0x0105,  # LATIN SMALL LETTER A WITH OGONEK
    "approxequal": 0x2248,  # ALMOST EQUAL TO
    "aring": 0x00E5,  # LATIN SMALL LETTER A WITH RING ABOVE
    "aringacute": 0x01FB,  # LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
    "arrowboth": 0x2194,  # LEFT RIGHT ARROW
    "arrowdblboth": 0x21D4,  # LEFT RIGHT DOUBLE ARROW
    "arrowdbldown": 0x21D3,  # DOWNWARDS DOUBLE ARROW
    "arrowdblleft": 0x21D0,  # LEFTWARDS DOUBLE ARROW
    "arrowdblright": 0x21D2,  # RIGHTWARDS DOUBLE ARROW
    "arrowdblup": 0x21D1,  # UPWARDS DOUBLE ARROW
    "arrowdown": 0x2193,  # DOWNWARDS ARROW
    "arrowleft": 0x2190,  # LEFTWARDS ARROW
    "arrowright": 0x2192,  # RIGHTWARDS ARROW
    "arrowup": 0x2191,  # UPWARDS ARROW
    "arrowupdn": 0x2195,  # UP DOWN ARROW
    "arrowupdnbse": 0x21A8,  # UP DOWN ARROW WITH BASE
    "asciicircum": 0x005E,  # CIRCUMFLEX ACCENT
    "asciitilde": 0x007E,  # TILDE
    "asterisk": 0x002A,  # ASTERISK
    "asteriskmath": 0x2217,  # ASTERISK OPERATOR
    "at": 0x0040,  # COMMERCIAL AT
    "atilde": 0x00E3,  # LATIN SMALL LETTER A WITH TILDE
    "b": 0x0062,  # LATIN SMALL LETTER B
    "backslash": 0x005C,  # REVERSE SOLIDUS
    "bar": 0x007C,  # VERTICAL LINE
    "beta": 0x03B2,  # GREEK SMALL LETTER BETA
    "block": 0x2588,  # FULL BLOCK
    "braceleft": 0x007B,  # LEFT CURLY BRACKET
    "braceright": 0x007D,  # RIGHT CURLY BRACKET
    "bracketleft": 0x005B,  # LEFT SQUARE BRACKET
    "bracketright": 0x005D,  # RIGHT SQUARE BRACKET
    "breve": 0x02D8,  # BREVE
    "brokenbar": 0x00A6,  # BROKEN BAR
    "bullet": 0x2022,  # BULLET
    "c": 0x0063,  # LATIN SMALL LETTER C
    "cacute": 0x0107,  # LATIN SMALL LETTER C WITH ACUTE
    "caron": 0x02C7,  # CARON
    "carriagereturn": 0x21B5,  # DOWNWARDS ARROW WITH CORNER LEFTWARDS
    "ccaron": 0x010D,  # LATIN SMALL LETTER C WITH CARON
    "ccedilla": 0x00E7,  # LATIN SMALL LETTER C WITH CEDILLA
    "ccircumflex": 0x0109,  # LATIN SMALL LETTER C WITH CIRCUMFLEX
    "cdotaccent": 0x010B,  # LATIN SMALL LETTER C WITH DOT ABOVE
    "cedilla": 0x00B8,  # CEDILLA
    "cent": 0x00A2,  # CENT SIGN
    "chi": 0x03C7,  # GREEK SMALL LETTER CHI
    "circle": 0x25CB,  # WHITE CIRCLE
    "circlemultiply": 0x2297,  # CIRCLED TIMES
    "circleplus": 0x2295,  # CIRCLED PLUS
    "circumflex": 0x02C6,  # MODIFIER LETTER CIRCUMFLEX ACCENT
    "club": 0x2663,  # BLACK CLUB SUIT
    "colon": 0x003A,  # COLON
    "colonmonetary": 0x20A1,  # COLON SIGN
    "comma": 0x002C,  # COMMA
    "congruent": 0x2245,  # APPROXIMATELY EQUAL TO
    "copyright": 0x00A9,  # COPYRIGHT SIGN
    "currency": 0x00A4,  # CURRENCY SIGN
    "d": 0x0064,  # LATIN SMALL LETTER D
    "dagger": 0x2020,  # DAGGER
    "daggerdbl": 0x2021,  # DOUBLE DAGGER
    "dcaron": 0x010F,  # LATIN SMALL LETTER D WITH CARON
    "dcroat": 0x0111,  # LATIN SMALL LETTER D WITH STROKE
    "degree": 0x00B0,  # DEGREE SIGN
    "delta": 0x03B4,  # GREEK SMALL LETTER DELTA
    "diamond": 0x2666,  # BLACK DIAMOND SUIT
    "dieresis": 0x00A8,  # DIAERESIS
    "dieresistonos": 0x0385,  # GREEK DIALYTIKA TONOS
    "divide": 0x00F7,  # DIVISION SIGN
    "dkshade": 0x2593,  # DARK SHADE
    "dnblock": 0x2584,  # LOWER HALF BLOCK
    "dollar": 0x0024,  # DOLLAR SIGN
    "dong": 0x20AB,  # DONG SIGN
    "dotaccent": 0x02D9,  # DOT ABOVE
    "dotbelowcomb": 0x0323,  # COMBINING DOT BELOW
    "dotlessi": 0x0131,  # LATIN SMALL LETTER DOTLESS I
    "dotmath": 0x22C5,  # DOT OPERATOR
    "e": 0x0065,  # LATIN SMALL LETTER E
    "eacute": 0x00E9,  # LATIN SMALL LETTER E WITH ACUTE
    "ebreve": 0x0115,  # LATIN SMALL LETTER E WITH BREVE
    "ecaron": 0x011B,  # LATIN SMALL LETTER E WITH CARON
    "ecircumflex": 0x00EA,  # LATIN SMALL LETTER E WITH CIRCUMFLEX
    "edieresis": 0x00EB,  # LATIN SMALL LETTER E WITH DIAERESIS
    "edotaccent": 0x0117,  # LATIN SMALL LETTER E WITH DOT ABOVE
    "egrave": 0x00E8,  # LATIN SMALL LETTER E WITH GRAVE
    "eight": 0x0038,  # DIGIT EIGHT
    "element": 0x2208,  # ELEMENT OF
    "ellipsis": 0x2026,  # HORIZONTAL ELLIPSIS
    "emacron": 0x0113,  # LATIN SMALL LETTER E WITH MACRON
    "emdash": 0x2014,  # EM DASH
    "emptyset": 0x2205,  # EMPTY SET
    "endash": 0x2013,  # EN DASH
    "eng": 0x014B,  # LATIN SMALL LETTER ENG
    "eogonek": 0x0119,  # LATIN SMALL LETTER E WITH OGONEK
    "epsilon": 0x03B5,  # GREEK SMALL LETTER EPSILON
    "epsilontonos": 0x03AD,  # GREEK SMALL LETTER EPSILON WITH TONOS
    "equal": 0x003D,  # EQUALS SIGN
    "equivalence": 0x2261,  # IDENTICAL TO
    "estimated": 0x212E,  # ESTIMATED SYMBOL
    "eta": 0x03B7,  # GREEK SMALL LETTER ETA
    "etatonos": 0x03AE,  # GREEK SMALL LETTER ETA WITH TONOS
    "eth": 0x00F0,  # LATIN SMALL LETTER ETH
    "exclam": 0x0021,  # EXCLAMATION MARK
    "exclamdbl": 0x203C,  # DOUBLE EXCLAMATION MARK
    "exclamdown": 0x00A1,  # INVERTED EXCLAMATION MARK
    "existential": 0x2203,  # THERE EXISTS
    "f": 0x0066,  # LATIN SMALL LETTER F
    "female": 0x2640,  # FEMALE SIGN
    "figuredash": 0x2012,  # FIGURE DASH
    "filledbox": 0x25A0,  # BLACK SQUARE
    "filledrect": 0x25AC,  # BLACK RECTANGLE
    "five": 0x0035,  # DIGIT FIVE
    "fiveeighths": 0x215D,  # VULGAR FRACTION FIVE EIGHTHS
    "florin": 0x0192,  # LATIN SMALL LETTER F WITH HOOK
    "four": 0x0034,  # DIGIT FOUR
    "fraction": 0x2044,  # FRACTION SLASH
    "franc": 0x20A3,  # FRENCH FRANC SIGN
    "g": 0x0067,  # LATIN SMALL LETTER G
    "gamma": 0x03B3,  # GREEK SMALL LETTER GAMMA
    "gbreve": 0x011F,  # LATIN SMALL LETTER G WITH BREVE
    "gcaron": 0x01E7,  # LATIN SMALL LETTER G WITH CARON
    "gcircumflex": 0x011D,  # LATIN SMALL LETTER G WITH CIRCUMFLEX
    "gdotaccent": 0x0121,  # LATIN SMALL LETTER G WITH DOT ABOVE
    "germandbls": 0x00DF,  # LATIN SMALL LETTER SHARP S
    "gradient": 0x2207,  # NABLA
    "grave": 0x0060,  # GRAVE ACCENT
    "gravecomb": 0x0300,  # COMBINING GRAVE ACCENT
    "greater": 0x003E,  # GREATER-THAN SIGN
    "greaterequal": 0x2265,  # GREATER-THAN OR EQUAL TO
    "guillemotleft": 0x00AB,  # LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
    "guillemotright": 0x00BB,  # RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
    "guilsinglleft": 0x2039,  # SINGLE LEFT-POINTING ANGLE QUOTATION MARK
    "guilsinglright": 0x203A,  # SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
    "h": 0x0068,  # LATIN SMALL LETTER H
    "hbar": 0x0127,  # LATIN SMALL LETTER H WITH STROKE
    "hcircumflex": 0x0125,  # LATIN SMALL LETTER H WITH CIRCUMFLEX
    "heart": 0x2665,  # BLACK HEART SUIT
    "hookabovecomb": 0x0309,  # COMBINING HOOK ABOVE
    "house": 0x2302,  # HOUSE
    "hungarumlaut": 0x02DD,  # DOUBLE ACUTE ACCENT
    "hyphen": 0x002D,  # HYPHEN-MINUS
    "i": 0x0069,  # LATIN SMALL LETTER I
    "iacute": 0x00ED,  # LATIN SMALL LETTER I WITH ACUTE
    "ibreve": 0x012D,  # LATIN SMALL LETTER I WITH BREVE
    "icircumflex": 0x00EE,  # LATIN SMALL LETTER I WITH CIRCUMFLEX
    "idieresis": 0x00EF,  # LATIN SMALL LETTER I WITH DIAERESIS
    "igrave": 0x00EC,  # LATIN SMALL LETTER I WITH GRAVE
    "ij": 0x0133,  # LATIN SMALL LIGATURE IJ
    "imacron": 0x012B,  # LATIN SMALL LETTER I WITH MACRON
    "infinity": 0x221E,  # INFINITY
    "integral": 0x222B,  # INTEGRAL
    "integralbt": 0x2321,  # BOTTOM HALF INTEGRAL
    "integraltp": 0x2320,  # TOP HALF INTEGRAL
    "intersection": 0x2229,  # INTERSECTION
    "invbullet": 0x25D8,  # INVERSE BULLET
    "invcircle": 0x25D9,  # INVERSE WHITE CIRCLE
    "invsmileface": 0x263B,  # BLACK SMILING FACE
    "iogonek": 0x012F,  # LATIN SMALL LETTER I WITH OGONEK
    "iota": 0x03B9,  # GREEK SMALL LETTER IOTA
    "iotadieresis": 0x03CA,  # GREEK SMALL LETTER IOTA WITH DIALYTIKA
    "iotadieresistonos": 0x0390,  # GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS
    "iotatonos": 0x03AF,  # GREEK SMALL LETTER IOTA WITH TONOS
    "itilde": 0x0129,  # LATIN SMALL LETTER I WITH TILDE
    "j": 0x006A,  # LATIN SMALL LETTER J
    "jcircumflex": 0x0135,  # LATIN SMALL LETTER J WITH CIRCUMFLEX
    "k": 0x006B,  # LATIN SMALL LETTER K
    "kappa": 0x03BA,  # GREEK SMALL LETTER KAPPA
    "kgreenlandic": 0x0138,  # LATIN SMALL LETTER KRA
    "l": 0x006C,  # LATIN SMALL LETTER L
    "lacute": 0x013A,  # LATIN SMALL LETTER L WITH ACUTE
    "lambda": 0x03BB,  # GREEK SMALL LETTER LAMDA
    "lcaron": 0x013E,  # LATIN SMALL LETTER L WITH CARON
    "ldot": 0x0140,  # LATIN SMALL LETTER L WITH MIDDLE DOT
    "less": 0x003C,  # LESS-THAN SIGN
    "lessequal": 0x2264,  # LESS-THAN OR EQUAL TO
    "lfblock": 0x258C,  # LEFT HALF BLOCK
    "lira": 0x20A4,  # LIRA SIGN
    "logicaland": 0x2227,  # LOGICAL AND
    "logicalnot": 0x00AC,  # NOT SIGN
    "logicalor": 0x2228,  # LOGICAL OR
    "longs": 0x017F,  # LATIN SMALL LETTER LONG S
    "lozenge": 0x25CA,  # LOZENGE
    "lslash": 0x0142,  # LATIN SMALL LETTER L WITH STROKE
    "ltshade": 0x2591,  # LIGHT SHADE
    "m": 0x006D,  # LATIN SMALL LETTER M
    "macron": 0x00AF,  # MACRON
    "male": 0x2642,  # MALE SIGN
    "minus": 0x2212,  # MINUS SIGN
    "minute": 0x2032,  # PRIME
    "mu": 0x00B5,  # MICRO SIGN
    "multiply": 0x00D7,  # MULTIPLICATION SIGN
    "musicalnote": 0x266A,  # EIGHTH NOTE
    "musicalnotedbl": 0x266B,  # BEAMED EIGHTH NOTES
    "n": 0x006E,  # LATIN SMALL LETTER N
    "nacute": 0x0144,  # LATIN SMALL LETTER N WITH ACUTE
    "napostrophe": 0x0149,  # LATIN SMALL LETTER N PRECEDED BY APOSTROPHE
    "ncaron": 0x0148,  # LATIN SMALL LETTER N WITH CARON
    "nine": 0x0039,  # DIGIT NINE
    "notelement": 0x2209,  # NOT AN ELEMENT OF
    "notequal": 0x2260,  # NOT EQUAL TO
    "notsubset": 0x2284,  # NOT A SUBSET OF
    "ntilde": 0x00F1,  # LATIN SMALL LETTER N WITH TILDE
    "nu": 0x03BD,  # GREEK SMALL LETTER NU
    "numbersign": 0x0023,  # NUMBER SIGN
    "o": 0x006F,  # LATIN SMALL LETTER O
    "oacute": 0x00F3,  # LATIN SMALL LETTER O WITH ACUTE
    "obreve": 0x014F,  # LATIN SMALL LETTER O WITH BREVE
    "ocircumflex": 0x00F4,  # LATIN SMALL LETTER O WITH CIRCUMFLEX
    "odieresis": 0x00F6,  # LATIN SMALL LETTER O WITH DIAERESIS
    "oe": 0x0153,  # LATIN SMALL LIGATURE OE
    "ogonek": 0x02DB,  # OGONEK
    "ograve": 0x00F2,  # LATIN SMALL LETTER O WITH GRAVE
    "ohorn": 0x01A1,  # LATIN SMALL LETTER O WITH HORN
    "ohungarumlaut": 0x0151,  # LATIN SMALL LETTER O WITH DOUBLE ACUTE
    "omacron": 0x014D,  # LATIN SMALL LETTER O WITH MACRON
    "omega": 0x03C9,  # GREEK SMALL LETTER OMEGA
    "omega1": 0x03D6,  # GREEK PI SYMBOL
    "omegatonos": 0x03CE,  # GREEK SMALL LETTER OMEGA WITH TONOS
    "omicron": 0x03BF,  # GREEK SMALL LETTER OMICRON
    "omicrontonos": 0x03CC,  # GREEK SMALL LETTER OMICRON WITH TONOS
    "one": 0x0031,  # DIGIT ONE
    "onedotenleader": 0x2024,  # ONE DOT LEADER
    "oneeighth": 0x215B,  # VULGAR FRACTION ONE EIGHTH
    "onehalf": 0x00BD,  # VULGAR FRACTION ONE HALF
    "onequarter": 0x00BC,  # VULGAR FRACTION ONE QUARTER
    "onethird": 0x2153,  # VULGAR FRACTION ONE THIRD
    "openbullet": 0x25E6,  # WHITE BULLET
    "ordfeminine": 0x00AA,  # FEMININE ORDINAL INDICATOR
    "ordmasculine": 0x00BA,  # MASCULINE ORDINAL INDICATOR
    "orthogonal": 0x221F,  # RIGHT ANGLE
    "oslash": 0x00F8,  # LATIN SMALL LETTER O WITH STROKE
    "oslashacute": 0x01FF,  # LATIN SMALL LETTER O WITH STROKE AND ACUTE
    "otilde": 0x00F5,  # LATIN SMALL LETTER O WITH TILDE
    "p": 0x0070,  # LATIN SMALL LETTER P
    "paragraph": 0x00B6,  # PILCROW SIGN
    "parenleft": 0x0028,  # LEFT PARENTHESIS
    "parenright": 0x0029,  # RIGHT PARENTHESIS
    "partialdiff": 0x2202,  # PARTIAL DIFFERENTIAL
    "percent": 0x0025,  # PERCENT SIGN
    "period": 0x002E,  # FULL STOP
    "periodcentered": 0x00B7,  # MIDDLE DOT
    "perpendicular": 0x22A5,  # UP TACK
    "perthousand": 0x2030,  # PER MILLE SIGN
    "peseta": 0x20A7,  # PESETA SIGN
    "phi": 0x03C6,  # GREEK SMALL LETTER PHI
    "phi1": 0x03D5,  # GREEK PHI SYMBOL
    "pi": 0x03C0,  # GREEK SMALL LETTER PI
    "plus": 0x002B,  # PLUS SIGN
    "plusminus": 0x00B1,  # PLUS-MINUS SIGN
    "prescription": 0x211E,  # PRESCRIPTION TAKE
    "product": 0x220F,  # N-ARY PRODUCT
    "propersubset": 0x2282,  # SUBSET OF
    "propersuperset": 0x2283,  # SUPERSET OF
    "proportional": 0x221D,  # PROPORTIONAL TO
    "psi": 0x03C8,  # GREEK SMALL LETTER PSI
    "q": 0x0071,  # LATIN SMALL LETTER Q
    "question": 0x003F,  # QUESTION MARK
    "questiondown": 0x00BF,  # INVERTED QUESTION MARK
    "quotedbl": 0x0022,  # QUOTATION MARK
    "quotedblbase": 0x201E,  # DOUBLE LOW-9 QUOTATION MARK
    "quotedblleft": 0x201C,  # LEFT DOUBLE QUOTATION MARK
    "quotedblright": 0x201D,  # RIGHT DOUBLE QUOTATION MARK
    "quoteleft": 0x2018,  # LEFT SINGLE QUOTATION MARK
    "quotereversed": 0x201B,  # SINGLE HIGH-REVERSED-9 QUOTATION MARK
    "quoteright": 0x2019,  # RIGHT SINGLE QUOTATION MARK
    "quotesinglbase": 0x201A,  # SINGLE LOW-9 QUOTATION MARK
    "quotesingle": 0x0027,  # APOSTROPHE
    "r": 0x0072,  # LATIN SMALL LETTER R
    "racute": 0x0155,  # LATIN SMALL LETTER R WITH ACUTE
    "radical": 0x221A,  # SQUARE ROOT
    "rcaron": 0x0159,  # LATIN SMALL LETTER R WITH CARON
    "reflexsubset": 0x2286,  # SUBSET OF OR EQUAL TO
    "reflexsuperset": 0x2287,  # SUPERSET OF OR EQUAL TO
    "registered": 0x00AE,  # REGISTERED SIGN
    "revlogicalnot": 0x2310,  # REVERSED NOT SIGN
    "rho": 0x03C1,  # GREEK SMALL LETTER RHO
    "ring": 0x02DA,  # RING ABOVE
    "rtblock": 0x2590,  # RIGHT HALF BLOCK
    "s": 0x0073,  # LATIN SMALL LETTER S
    "sacute": 0x015B,  # LATIN SMALL LETTER S WITH ACUTE
    "scaron": 0x0161,  # LATIN SMALL LETTER S WITH CARON
    "scedilla": 0x015F,  # LATIN SMALL LETTER S WITH CEDILLA
    "scircumflex": 0x015D,  # LATIN SMALL LETTER S WITH CIRCUMFLEX
    "second": 0x2033,  # DOUBLE PRIME
    "section": 0x00A7,  # SECTION SIGN
    "semicolon": 0x003B,  # SEMICOLON
    "seven": 0x0037,  # DIGIT SEVEN
    "seveneighths": 0x215E,  # VULGAR FRACTION SEVEN EIGHTHS
    "shade": 0x2592,  # MEDIUM SHADE
    "sigma": 0x03C3,  # GREEK SMALL LETTER SIGMA
    "sigma1": 0x03C2,  # GREEK SMALL LETTER FINAL SIGMA
    "similar": 0x223C,  # TILDE OPERATOR
    "six": 0x0036,  # DIGIT SIX
    "slash": 0x002F,  # SOLIDUS
    "smileface": 0x263A,  # WHITE SMILING FACE
    "space": 0x0020,  # SPACE
    "spade": 0x2660,  # BLACK SPADE SUIT
    "sterling": 0x00A3,  # POUND SIGN
    "suchthat": 0x220B,  # CONTAINS AS MEMBER
    "summation": 0x2211,  # N-ARY SUMMATION
    "sun": 0x263C,  # WHITE SUN WITH RAYS
    "t": 0x0074,  # LATIN SMALL LETTER T
    "tau": 0x03C4,  # GREEK SMALL LETTER TAU
    "tbar": 0x0167,  # LATIN SMALL LETTER T WITH STROKE
    "tcaron": 0x0165,  # LATIN SMALL LETTER T WITH CARON
    "therefore": 0x2234,  # THEREFORE
    "theta": 0x03B8,  # GREEK SMALL LETTER THETA
    "theta1": 0x03D1,  # GREEK THETA SYMBOL
    "thorn": 0x00FE,  # LATIN SMALL LETTER THORN
    "three": 0x0033,  # DIGIT THREE
    "threeeighths": 0x215C,  # VULGAR FRACTION THREE EIGHTHS
    "threequarters": 0x00BE,  # VULGAR FRACTION THREE QUARTERS
    "tilde": 0x02DC,  # SMALL TILDE
    "tildecomb": 0x0303,  # COMBINING TILDE
    "tonos": 0x0384,  # GREEK TONOS
    "trademark": 0x2122,  # TRADE MARK SIGN
    "triagdn": 0x25BC,  # BLACK DOWN-POINTING TRIANGLE
    "triaglf": 0x25C4,  # BLACK LEFT-POINTING POINTER
    "triagrt": 0x25BA,  # BLACK RIGHT-POINTING POINTER
    "triagup": 0x25B2,  # BLACK UP-POINTING TRIANGLE
    "two": 0x0032,  # DIGIT TWO
    "twodotenleader": 0x2025,  # TWO DOT LEADER
    "twothirds": 0x2154,  # VULGAR FRACTION TWO THIRDS
    "u": 0x0075,  # LATIN SMALL LETTER U
    "uacute": 0x00FA,  # LATIN SMALL LETTER U WITH ACUTE
    "ubreve": 0x016D,  # LATIN SMALL LETTER U WITH BREVE
    "ucircumflex": 0x00FB,  # LATIN SMALL LETTER U WITH CIRCUMFLEX
    "udieresis": 0x00FC,  # LATIN SMALL LETTER U WITH DIAERESIS
    "ugrave": 0x00F9,  # LATIN SMALL LETTER U WITH GRAVE
    "uhorn": 0x01B0,  # LATIN SMALL LETTER U WITH HORN
    "uhungarumlaut": 0x0171,  # LATIN SMALL LETTER U WITH DOUBLE ACUTE
    "umacron": 0x016B,  # LATIN SMALL LETTER U WITH MACRON
    "underscore": 0x005F,  # LOW LINE
    "underscoredbl": 0x2017,  # DOUBLE LOW LINE
    "union": 0x222A,  # UNION
    "universal": 0x2200,  # FOR ALL
    "uogonek": 0x0173,  # LATIN SMALL LETTER U WITH OGONEK
    "upblock": 0x2580,  # UPPER HALF BLOCK
    "upsilon": 0x03C5,  # GREEK SMALL LETTER UPSILON
    "upsilondieresis": 0x03CB,  # GREEK SMALL LETTER UPSILON WITH DIALYTIKA
    "upsilondieresistonos": 0x03B0,  # GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
    "upsilontonos": 0x03CD,  # GREEK SMALL LETTER UPSILON WITH TONOS
    "uring": 0x016F,  # LATIN SMALL LETTER U WITH RING ABOVE
    "utilde": 0x0169,  # LATIN SMALL LETTER U WITH TILDE
    "v": 0x0076,  # LATIN SMALL LETTER V
    "w": 0x0077,  # LATIN SMALL LETTER W
    "wacute": 0x1E83,  # LATIN SMALL LETTER W WITH ACUTE
    "wcircumflex": 0x0175,  # LATIN SMALL LETTER W WITH CIRCUMFLEX
    "wdieresis": 0x1E85,  # LATIN SMALL LETTER W WITH DIAERESIS
    "weierstrass": 0x2118,  # SCRIPT CAPITAL P
    "wgrave": 0x1E81,  # LATIN SMALL LETTER W WITH GRAVE
    "x": 0x0078,  # LATIN SMALL LETTER X
    "xi": 0x03BE,  # GREEK SMALL LETTER XI
    "y": 0x0079,  # LATIN SMALL LETTER Y
    "yacute": 0x00FD,  # LATIN SMALL LETTER Y WITH ACUTE
    "ycircumflex": 0x0177,  # LATIN SMALL LETTER Y WITH CIRCUMFLEX
    "ydieresis": 0x00FF,  # LATIN SMALL LETTER Y WITH DIAERESIS
    "yen": 0x00A5,  # YEN SIGN
    "ygrave": 0x1EF3,  # LATIN SMALL LETTER Y WITH GRAVE
    "z": 0x007A,  # LATIN SMALL LETTER Z
    "zacute": 0x017A,  # LATIN SMALL LETTER Z WITH ACUTE
    "zcaron": 0x017E,  # LATIN SMALL LETTER Z WITH CARON
    "zdotaccent": 0x017C,  # LATIN SMALL LETTER Z WITH DOT ABOVE
    "zero": 0x0030,  # DIGIT ZERO
    "zeta": 0x03B6,  # GREEK SMALL LETTER ZETA
    # Local additions:
    "CR": 0x000D,
    "NULL": 0x0000,
    "fi": 0xFB01,
    "fl": 0xFB02,
    "onesuperior": 0x00B9,
    "threesuperior": 0x00B3,
    "twosuperior": 0x00B2,
}



# This is a generated file, use data-scripts/generatePyUniData.py to edit and regenerate.

uniBlocks = {
    (0x0000, 0x007F): "Basic Latin",
    (0x0080, 0x00FF): "Latin-1 Supplement",
    (0x0100, 0x017F): "Latin Extended-A",
    (0x0180, 0x024F): "Latin Extended-B",
    (0x0250, 0x02AF): "IPA Extensions",
    (0x02B0, 0x02FF): "Spacing Modifier Letters",
    (0x0300, 0x036F): "Combining Diacritical Marks",
    (0x0370, 0x03FF): "Greek and Coptic",
    (0x0400, 0x04FF): "Cyrillic",
    (0x0500, 0x052F): "Cyrillic Supplement",
    (0x0530, 0x058F): "Armenian",
    (0x0590, 0x05FF): "Hebrew",
    (0x0600, 0x06FF): "Arabic",
    (0x0700, 0x074F): "Syriac",
    (0x0750, 0x077F): "Arabic Supplement",
    (0x0780, 0x07BF): "Thaana",
    (0x07C0, 0x07FF): "NKo",
    (0x0800, 0x083F): "Samaritan",
    (0x0840, 0x085F): "Mandaic",
    (0x0860, 0x086F): "Syriac Supplement",
    (0x0870, 0x089F): "Arabic Extended-B",
    (0x08A0, 0x08FF): "Arabic Extended-A",
    (0x0900, 0x097F): "Devanagari",
    (0x0980, 0x09FF): "Bengali",
    (0x0A00, 0x0A7F): "Gurmukhi",
    (0x0A80, 0x0AFF): "Gujarati",
    (0x0B00, 0x0B7F): "Oriya",
    (0x0B80, 0x0BFF): "Tamil",
    (0x0C00, 0x0C7F): "Telugu",
    (0x0C80, 0x0CFF): "Kannada",
    (0x0D00, 0x0D7F): "Malayalam",
    (0x0D80, 0x0DFF): "Sinhala",
    (0x0E00, 0x0E7F): "Thai",
    (0x0E80, 0x0EFF): "Lao",
    (0x0F00, 0x0FFF): "Tibetan",
    (0x1000, 0x109F): "Myanmar",
    (0x10A0, 0x10FF): "Georgian",
    (0x1100, 0x11FF): "Hangul Jamo",
    (0x1200, 0x137F): "Ethiopic",
    (0x1380, 0x139F): "Ethiopic Supplement",
    (0x13A0, 0x13FF): "Cherokee",
    (0x1400, 0x167F): "Unified Canadian Aboriginal Syllabics",
    (0x1680, 0x169F): "Ogham",
    (0x16A0, 0x16FF): "Runic",
    (0x1700, 0x171F): "Tagalog",
    (0x1720, 0x173F): "Hanunoo",
    (0x1740, 0x175F): "Buhid",
    (0x1760, 0x177F): "Tagbanwa",
    (0x1780, 0x17FF): "Khmer",
    (0x1800, 0x18AF): "Mongolian",
    (0x18B0, 0x18FF): "Unified Canadian Aboriginal Syllabics Extended",
    (0x1900, 0x194F): "Limbu",
    (0x1950, 0x197F): "Tai Le",
    (0x1980, 0x19DF): "New Tai Lue",
    (0x19E0, 0x19FF): "Khmer Symbols",
    (0x1A00, 0x1A1F): "Buginese",
    (0x1A20, 0x1AAF): "Tai Tham",
    (0x1AB0, 0x1AFF): "Combining Diacritical Marks Extended",
    (0x1B00, 0x1B7F): "Balinese",
    (0x1B80, 0x1BBF): "Sundanese",
    (0x1BC0, 0x1BFF): "Batak",
    (0x1C00, 0x1C4F): "Lepcha",
    (0x1C50, 0x1C7F): "Ol Chiki",
    (0x1C80, 0x1C8F): "Cyrillic Extended-C",
    (0x1C90, 0x1CBF): "Georgian Extended",
    (0x1CC0, 0x1CCF): "Sundanese Supplement",
    (0x1CD0, 0x1CFF): "Vedic Extensions",
    (0x1D00, 0x1D7F): "Phonetic Extensions",
    (0x1D80, 0x1DBF): "Phonetic Extensions Supplement",
    (0x1DC0, 0x1DFF): "Combining Diacritical Marks Supplement",
    (0x1E00, 0x1EFF): "Latin Extended Additional",
    (0x1F00, 0x1FFF): "Greek Extended",
    (0x2000, 0x206F): "General Punctuation",
    (0x2070, 0x209F): "Superscripts and Subscripts",
    (0x20A0, 0x20CF): "Currency Symbols",
    (0x20D0, 0x20FF): "Combining Diacritical Marks for Symbols",
    (0x2100, 0x214F): "Letterlike Symbols",
    (0x2150, 0x218F): "Number Forms",
    (0x2190, 0x21FF): "Arrows",
    (0x2200, 0x22FF): "Mathematical Operators",
    (0x2300, 0x23FF): "Miscellaneous Technical",
    (0x2400, 0x243F): "Control Pictures",
    (0x2440, 0x245F): "Optical Character Recognition",
    (0x2460, 0x24FF): "Enclosed Alphanumerics",
    (0x2500, 0x257F): "Box Drawing",
    (0x2580, 0x259F): "Block Elements",
    (0x25A0, 0x25FF): "Geometric Shapes",
    (0x2600, 0x26FF): "Miscellaneous Symbols",
    (0x2700, 0x27BF): "Dingbats",
    (0x27C0, 0x27EF): "Miscellaneous Mathematical Symbols-A",
    (0x27F0, 0x27FF): "Supplemental Arrows-A",
    (0x2800, 0x28FF): "Braille Patterns",
    (0x2900, 0x297F): "Supplemental Arrows-B",
    (0x2980, 0x29FF): "Miscellaneous Mathematical Symbols-B",
    (0x2A00, 0x2AFF): "Supplemental Mathematical Operators",
    (0x2B00, 0x2BFF): "Miscellaneous Symbols and Arrows",
    (0x2C00, 0x2C5F): "Glagolitic",
    (0x2C60, 0x2C7F): "Latin Extended-C",
    (0x2C80, 0x2CFF): "Coptic",
    (0x2D00, 0x2D2F): "Georgian Supplement",
    (0x2D30, 0x2D7F): "Tifinagh",
    (0x2D80, 0x2DDF): "Ethiopic Extended",
    (0x2DE0, 0x2DFF): "Cyrillic Extended-A",
    (0x2E00, 0x2E7F): "Supplemental Punctuation",
    (0x2E80, 0x2EFF): "CJK Radicals Supplement",
    (0x2F00, 0x2FDF): "Kangxi Radicals",
    (0x2FF0, 0x2FFF): "Ideographic Description Characters",
    (0x3000, 0x303F): "CJK Symbols and Punctuation",
    (0x3040, 0x309F): "Hiragana",
    (0x30A0, 0x30FF): "Katakana",
    (0x3100, 0x312F): "Bopomofo",
    (0x3130, 0x318F): "Hangul Compatibility Jamo",
    (0x3190, 0x319F): "Kanbun",
    (0x31A0, 0x31BF): "Bopomofo Extended",
    (0x31C0, 0x31EF): "CJK Strokes",
    (0x31F0, 0x31FF): "Katakana Phonetic Extensions",
    (0x3200, 0x32FF): "Enclosed CJK Letters and Months",
    (0x3300, 0x33FF): "CJK Compatibility",
    (0x3400, 0x4DBF): "CJK Unified Ideographs Extension A",
    (0x4DC0, 0x4DFF): "Yijing Hexagram Symbols",
    (0x4E00, 0x9FFF): "CJK Unified Ideographs",
    (0xA000, 0xA48F): "Yi Syllables",
    (0xA490, 0xA4CF): "Yi Radicals",
    (0xA4D0, 0xA4FF): "Lisu",
    (0xA500, 0xA63F): "Vai",
    (0xA640, 0xA69F): "Cyrillic Extended-B",
    (0xA6A0, 0xA6FF): "Bamum",
    (0xA700, 0xA71F): "Modifier Tone Letters",
    (0xA720, 0xA7FF): "Latin Extended-D",
    (0xA800, 0xA82F): "Syloti Nagri",
    (0xA830, 0xA83F): "Common Indic Number Forms",
    (0xA840, 0xA87F): "Phags-pa",
    (0xA880, 0xA8DF): "Saurashtra",
    (0xA8E0, 0xA8FF): "Devanagari Extended",
    (0xA900, 0xA92F): "Kayah Li",
    (0xA930, 0xA95F): "Rejang",
    (0xA960, 0xA97F): "Hangul Jamo Extended-A",
    (0xA980, 0xA9DF): "Javanese",
    (0xA9E0, 0xA9FF): "Myanmar Extended-B",
    (0xAA00, 0xAA5F): "Cham",
    (0xAA60, 0xAA7F): "Myanmar Extended-A",
    (0xAA80, 0xAADF): "Tai Viet",
    (0xAAE0, 0xAAFF): "Meetei Mayek Extensions",
    (0xAB00, 0xAB2F): "Ethiopic Extended-A",
    (0xAB30, 0xAB6F): "Latin Extended-E",
    (0xAB70, 0xABBF): "Cherokee Supplement",
    (0xABC0, 0xABFF): "Meetei Mayek",
    (0xAC00, 0xD7AF): "Hangul Syllables",
    (0xD7B0, 0xD7FF): "Hangul Jamo Extended-B",
    (0xD800, 0xDB7F): "High Surrogates",
    (0xDB80, 0xDBFF): "High Private Use Surrogates",
    (0xDC00, 0xDFFF): "Low Surrogates",
    (0xE000, 0xF8FF): "Private Use Area",
    (0xF900, 0xFAFF): "CJK Compatibility Ideographs",
    (0xFB00, 0xFB4F): "Alphabetic Presentation Forms",
    (0xFB50, 0xFDFF): "Arabic Presentation Forms-A",
    (0xFE00, 0xFE0F): "Variation Selectors",
    (0xFE10, 0xFE1F): "Vertical Forms",
    (0xFE20, 0xFE2F): "Combining Half Marks",
    (0xFE30, 0xFE4F): "CJK Compatibility Forms",
    (0xFE50, 0xFE6F): "Small Form Variants",
    (0xFE70, 0xFEFF): "Arabic Presentation Forms-B",
    (0xFF00, 0xFFEF): "Halfwidth and Fullwidth Forms",
    (0xFFF0, 0xFFFF): "Specials",
    (0x10000, 0x1007F): "Linear B Syllabary",
    (0x10080, 0x100FF): "Linear B Ideograms",
    (0x10100, 0x1013F): "Aegean Numbers",
    (0x10140, 0x1018F): "Ancient Greek Numbers",
    (0x10190, 0x101CF): "Ancient Symbols",
    (0x101D0, 0x101FF): "Phaistos Disc",
    (0x10280, 0x1029F): "Lycian",
    (0x102A0, 0x102DF): "Carian",
    (0x102E0, 0x102FF): "Coptic Epact Numbers",
    (0x10300, 0x1032F): "Old Italic",
    (0x10330, 0x1034F): "Gothic",
    (0x10350, 0x1037F): "Old Permic",
    (0x10380, 0x1039F): "Ugaritic",
    (0x103A0, 0x103DF): "Old Persian",
    (0x10400, 0x1044F): "Deseret",
    (0x10450, 0x1047F): "Shavian",
    (0x10480, 0x104AF): "Osmanya",
    (0x104B0, 0x104FF): "Osage",
    (0x10500, 0x1052F): "Elbasan",
    (0x10530, 0x1056F): "Caucasian Albanian",
    (0x10570, 0x105BF): "Vithkuqi",
    (0x10600, 0x1077F): "Linear A",
    (0x10780, 0x107BF): "Latin Extended-F",
    (0x10800, 0x1083F): "Cypriot Syllabary",
    (0x10840, 0x1085F): "Imperial Aramaic",
    (0x10860, 0x1087F): "Palmyrene",
    (0x10880, 0x108AF): "Nabataean",
    (0x108E0, 0x108FF): "Hatran",
    (0x10900, 0x1091F): "Phoenician",
    (0x10920, 0x1093F): "Lydian",
    (0x10980, 0x1099F): "Meroitic Hieroglyphs",
    (0x109A0, 0x109FF): "Meroitic Cursive",
    (0x10A00, 0x10A5F): "Kharoshthi",
    (0x10A60, 0x10A7F): "Old South Arabian",
    (0x10A80, 0x10A9F): "Old North Arabian",
    (0x10AC0, 0x10AFF): "Manichaean",
    (0x10B00, 0x10B3F): "Avestan",
    (0x10B40, 0x10B5F): "Inscriptional Parthian",
    (0x10B60, 0x10B7F): "Inscriptional Pahlavi",
    (0x10B80, 0x10BAF): "Psalter Pahlavi",
    (0x10C00, 0x10C4F): "Old Turkic",
    (0x10C80, 0x10CFF): "Old Hungarian",
    (0x10D00, 0x10D3F): "Hanifi Rohingya",
    (0x10E60, 0x10E7F): "Rumi Numeral Symbols",
    (0x10E80, 0x10EBF): "Yezidi",
    (0x10EC0, 0x10EFF): "Arabic Extended-C",
    (0x10F00, 0x10F2F): "Old Sogdian",
    (0x10F30, 0x10F6F): "Sogdian",
    (0x10F70, 0x10FAF): "Old Uyghur",
    (0x10FB0, 0x10FDF): "Chorasmian",
    (0x10FE0, 0x10FFF): "Elymaic",
    (0x11000, 0x1107F): "Brahmi",
    (0x11080, 0x110CF): "Kaithi",
    (0x110D0, 0x110FF): "Sora Sompeng",
    (0x11100, 0x1114F): "Chakma",
    (0x11150, 0x1117F): "Mahajani",
    (0x11180, 0x111DF): "Sharada",
    (0x111E0, 0x111FF): "Sinhala Archaic Numbers",
    (0x11200, 0x1124F): "Khojki",
    (0x11280, 0x112AF): "Multani",
    (0x112B0, 0x112FF): "Khudawadi",
    (0x11300, 0x1137F): "Grantha",
    (0x11400, 0x1147F): "Newa",
    (0x11480, 0x114DF): "Tirhuta",
    (0x11580, 0x115FF): "Siddham",
    (0x11600, 0x1165F): "Modi",
    (0x11660, 0x1167F): "Mongolian Supplement",
    (0x11680, 0x116CF): "Takri",
    (0x11700, 0x1174F): "Ahom",
    (0x11800, 0x1184F): "Dogra",
    (0x118A0, 0x118FF): "Warang Citi",
    (0x11900, 0x1195F): "Dives Akuru",
    (0x119A0, 0x119FF): "Nandinagari",
    (0x11A00, 0x11A4F): "Zanabazar Square",
    (0x11A50, 0x11AAF): "Soyombo",
    (0x11AB0, 0x11ABF): "Unified Canadian Aboriginal Syllabics Extended-A",
    (0x11AC0, 0x11AFF): "Pau Cin Hau",
    (0x11B00, 0x11B5F): "Devanagari Extended-A",
    (0x11C00, 0x11C6F): "Bhaiksuki",
    (0x11C70, 0x11CBF): "Marchen",
    (0x11D00, 0x11D5F): "Masaram Gondi",
    (0x11D60, 0x11DAF): "Gunjala Gondi",
    (0x11EE0, 0x11EFF): "Makasar",
    (0x11F00, 0x11F5F): "Kawi",
    (0x11FB0, 0x11FBF): "Lisu Supplement",
    (0x11FC0, 0x11FFF): "Tamil Supplement",
    (0x12000, 0x123FF): "Cuneiform",
    (0x12400, 0x1247F): "Cuneiform Numbers and Punctuation",
    (0x12480, 0x1254F): "Early Dynastic Cuneiform",
    (0x12F90, 0x12FFF): "Cypro-Minoan",
    (0x13000, 0x1342F): "Egyptian Hieroglyphs",
    (0x13430, 0x1345F): "Egyptian Hieroglyph Format Controls",
    (0x14400, 0x1467F): "Anatolian Hieroglyphs",
    (0x16800, 0x16A3F): "Bamum Supplement",
    (0x16A40, 0x16A6F): "Mro",
    (0x16A70, 0x16ACF): "Tangsa",
    (0x16AD0, 0x16AFF): "Bassa Vah",
    (0x16B00, 0x16B8F): "Pahawh Hmong",
    (0x16E40, 0x16E9F): "Medefaidrin",
    (0x16F00, 0x16F9F): "Miao",
    (0x16FE0, 0x16FFF): "Ideographic Symbols and Punctuation",
    (0x17000, 0x187FF): "Tangut",
    (0x18800, 0x18AFF): "Tangut Components",
    (0x18B00, 0x18CFF): "Khitan Small Script",
    (0x18D00, 0x18D7F): "Tangut Supplement",
    (0x1AFF0, 0x1AFFF): "Kana Extended-B",
    (0x1B000, 0x1B0FF): "Kana Supplement",
    (0x1B100, 0x1B12F): "Kana Extended-A",
    (0x1B130, 0x1B16F): "Small Kana Extension",
    (0x1B170, 0x1B2FF): "Nushu",
    (0x1BC00, 0x1BC9F): "Duployan",
    (0x1BCA0, 0x1BCAF): "Shorthand Format Controls",
    (0x1CF00, 0x1CFCF): "Znamenny Musical Notation",
    (0x1D000, 0x1D0FF): "Byzantine Musical Symbols",
    (0x1D100, 0x1D1FF): "Musical Symbols",
    (0x1D200, 0x1D24F): "Ancient Greek Musical Notation",
    (0x1D2C0, 0x1D2DF): "Kaktovik Numerals",
    (0x1D2E0, 0x1D2FF): "Mayan Numerals",
    (0x1D300, 0x1D35F): "Tai Xuan Jing Symbols",
    (0x1D360, 0x1D37F): "Counting Rod Numerals",
    (0x1D400, 0x1D7FF): "Mathematical Alphanumeric Symbols",
    (0x1D800, 0x1DAAF): "Sutton SignWriting",
    (0x1DF00, 0x1DFFF): "Latin Extended-G",
    (0x1E000, 0x1E02F): "Glagolitic Supplement",
    (0x1E030, 0x1E08F): "Cyrillic Extended-D",
    (0x1E100, 0x1E14F): "Nyiakeng Puachue Hmong",
    (0x1E290, 0x1E2BF): "Toto",
    (0x1E2C0, 0x1E2FF): "Wancho",
    (0x1E4D0, 0x1E4FF): "Nag Mundari",
    (0x1E7E0, 0x1E7FF): "Ethiopic Extended-B",
    (0x1E800, 0x1E8DF): "Mende Kikakui",
    (0x1E900, 0x1E95F): "Adlam",
    (0x1EC70, 0x1ECBF): "Indic Siyaq Numbers",
    (0x1ED00, 0x1ED4F): "Ottoman Siyaq Numbers",
    (0x1EE00, 0x1EEFF): "Arabic Mathematical Alphabetic Symbols",
    (0x1F000, 0x1F02F): "Mahjong Tiles",
    (0x1F030, 0x1F09F): "Domino Tiles",
    (0x1F0A0, 0x1F0FF): "Playing Cards",
    (0x1F100, 0x1F1FF): "Enclosed Alphanumeric Supplement",
    (0x1F200, 0x1F2FF): "Enclosed Ideographic Supplement",
    (0x1F300, 0x1F5FF): "Miscellaneous Symbols and Pictographs",
    (0x1F600, 0x1F64F): "Emoticons",
    (0x1F650, 0x1F67F): "Ornamental Dingbats",
    (0x1F680, 0x1F6FF): "Transport and Map Symbols",
    (0x1F700, 0x1F77F): "Alchemical Symbols",
    (0x1F780, 0x1F7FF): "Geometric Shapes Extended",
    (0x1F800, 0x1F8FF): "Supplemental Arrows-C",
    (0x1F900, 0x1F9FF): "Supplemental Symbols and Pictographs",
    (0x1FA00, 0x1FA6F): "Chess Symbols",
    (0x1FA70, 0x1FAFF): "Symbols and Pictographs Extended-A",
    (0x1FB00, 0x1FBFF): "Symbols for Legacy Computing",
    (0x20000, 0x2A6DF): "CJK Unified Ideographs Extension B",
    (0x2A700, 0x2B73F): "CJK Unified Ideographs Extension C",
    (0x2B740, 0x2B81F): "CJK Unified Ideographs Extension D",
    (0x2B820, 0x2CEAF): "CJK Unified Ideographs Extension E",
    (0x2CEB0, 0x2EBEF): "CJK Unified Ideographs Extension F",
    (0x2F800, 0x2FA1F): "CJK Compatibility Ideographs Supplement",
    (0x30000, 0x3134F): "CJK Unified Ideographs Extension G",
    (0x31350, 0x323AF): "CJK Unified Ideographs Extension H",
    (0xE0000, 0xE007F): "Tags",
    (0xE0100, 0xE01EF): "Variation Selectors Supplement",
    (0xF0000, 0xFFFFF): "Supplementary Private Use Area-A",
    (0x100000, 0x10FFFF): "Supplementary Private Use Area-B",
}



from __future__ import annotations

from jkUnicode.uniBlock import get_block
from jkUnicode.uniNiceName import nice_name_rules
from jkUnicode.uniCat import uniCat
from jkUnicode.uniCase import uniUpperCaseMapping, uniLowerCaseMapping
from jkUnicode.uniDecomposition import uniDecompositionMapping
from jkUnicode.uniScript import get_script

from typing import List, Optional, Tuple


categoryName = {
    "Lu": "Letter, Uppercase",
    "Ll": "Letter, Lowercase",
    "Lt": "Letter, Titlecase",
    "Lm": "Letter, Modifier",
    "Lo": "Letter, Other",
    "Mn": "Mark, Nonspacing",
    "Mc": "Mark, Spacing Combining",
    "Me": "Mark, Enclosing",
    "Nd": "Number, Decimal Digit",
    "Nl": "Number, Letter",
    "No": "Number, Other",
    "Pc": "Punctuation, Connector",
    "Pd": "Punctuation, Dash",
    "Ps": "Punctuation, Open",
    "Pe": "Punctuation, Close",
    "Pi": "Punctuation, Initial quote (may behave like Ps or Pe depending on usage)",
    "Pf": "Punctuation, Final quote (may behave like Ps or Pe depending on usage)",
    "Po": "Punctuation, Other",
    "Sm": "Symbol, Math",
    "Sc": "Symbol, Currency",
    "Sk": "Symbol, Modifier",
    "So": "Symbol, Other",
    "Zs": "Separator, Space",
    "Zl": "Separator, Line",
    "Zp": "Separator, Paragraph",
    "Cc": "Other, Control",
    "Cf": "Other, Format",
    "Cs": "Other, Surrogate",
    "Co": "Other, Private Use",
    "Cn": "Other, Not Assigned",
}


def get_expanded_glyph_list(
    unicodes: List[int], ui: Optional["UniInfo"] = None
) -> List[Tuple[int, Optional[str]]]:
    """ "Expand" or annotate a list of codepoints.

    For codepoints that have a case mapping (UC or LC), the target codepoint of
    the case mapping will be added to the list. AGLFN glyph names are added to
    the list too, so the returned list contains tuples of `(codepoint,
    glyphname)`, sorted by the codepoint value.

    :param unicodes: A list of codepoints
    :type unicodes: list

    :param  ui: The UniInfo instance to use. If None, one will be instantiated.
    :type  ui: UniInfo"""
    glyphs = []
    if ui is None:
        ui = UniInfo(0)
    for ch in unicodes:
        ui.unicode = ch
        glyphs.append((ch, ui.glyphname))
        if ui.lc_mapping is not None:
            ui.unicode = ui.lc_mapping
            glyphs.append((ui.unicode, ui.glyphname))
        elif ui.uc_mapping is not None:
            ui.unicode = ui.uc_mapping
            glyphs.append((ui.unicode, ui.glyphname))
    return sorted(list(set(glyphs)))


def getUnicodeChar(code: int) -> str:
    """Return the Unicode character for a Unicode codepoint.

    :param code: The codepoint
    :type code: int"""
    return chr(code)


class UniInfo:
    """The main Unicode Info object. It gets its Unicode information from the
    submodules aglfn, uniCase, uniCat, uniDecomposition, uniName, and
    uniRangesBits which are generated from the official Unicode data. You can
    find tools to download and regenerate the data in the `tools` subfolder.
    """

    def __init__(self, uni: Optional[int] = None) -> None:
        """The Unicode Info object is meant to be instantiated once and then
        reused to get information about different codepoints. Avoid to
        instantiate it often, because it is rather expensive on disk access.

        Initialize the Info object with a dummy codepoint or None e.g. before a
        loop and then in the loop assign the actual codepoints that you want
        information about by setting the `unicode` instance variable. This will
        automatically update the other instance variables with the correct
        information from the Unicode standard.

        :param uni: The codepoint.
        :type uni: int"""
        self._load_uni_name()
        self.unicode: Optional[int] = uni

    def _load_uni_name(self, file_name: str = "uni_names") -> None:
        """Import uniName from JSON or from Python"""
        # Loading the data from JSON is not faster than importing a python
        # module. Skip the optimization.
        from jkUnicode.uniName import uniName

        self.uniName = uniName
        # from jkUnicode.tools.jsonhelpers import (
        #     dict_from_file,
        #     json_path,
        #     json_to_file
        # )
        # from time import time
        # try:
        #     start = time()
        #     self.uniName = {
        #         int(k): v
        #         for k, v in dict_from_file(json_path, file_name).items()
        #     }
        #     stop = time()
        #     print(f"Loaded Unicode Name data from JSON in {stop - start} s.")
        # except FileNotFoundError:
        #     start = time()
        #     from jkUnicode.uniName import uniName
        #     self.uniName = uniName
        #     stop = time()
        #     print(f"Converted Unicode Name data to JSON in {stop - start} s.")
        #     json_to_file(json_path, file_name, self.uniName)

    @property
    def unicode(self) -> Optional[int]:
        """The Unicode codepoint. Setting this value will look up and fill the
        other pieces of information, like category, range, decomposition
        mapping, and case mapping."""
        return self._unicode

    @unicode.setter
    def unicode(self, value: Optional[int]):
        self._unicode = value
        # Set some properties to None
        # Those are expensive to compute, we do it only when asked to.
        self._ublock: Optional[str] = None
        self._script: Optional[str] = None
        self._name: Optional[str] = None
        self._categoryShort: Optional[str] = None
        self._category: Optional[str] = None
        self._uc_mapping: Optional[int] = None
        self._lc_mapping: Optional[int] = None
        self._dc_mapping: Optional[List[int]] = None

        if self._unicode is None:
            self._categoryShort = "<undefined>"
            self._category = "<undefined>"
            self._dc_mapping = []

    def __repr__(self) -> str:
        if self.unicode is None:
            s = "      Unicode: None"
        else:
            s = "      Unicode: 0x%04X (dec. %s)" % (
                self.unicode,
                self.unicode,
            )
        s += "\n         Name: %s" % self.name
        s += "\n     Category: %s (%s)" % (self._categoryShort, self.category)
        if self._uc_mapping:
            s += "\n    Uppercase: 0x%04X" % self._uc_mapping
        if self._lc_mapping:
            s += "\n    Lowercase: 0x%04X" % self._lc_mapping
        if self._dc_mapping:
            s += "\nDecomposition: %s" % (
                " ".join(["0x%04X" % m for m in self._dc_mapping])
            )
        return s

    @property
    def block(self) -> Optional[str]:
        """The name of the block for the current codepoint."""
        if self._ublock is None:
            self._ublock = get_block(self._unicode)
        return self._ublock

    @property
    def category(self) -> Optional[str]:
        """The name of the category for the current codepoint."""
        if self._unicode is None:
            return None

        if self._category is None:
            cs = self.category_short
            if cs is None:
                self._category = "<undefined>"
            else:
                self._category = categoryName.get(cs, "<undefined>")
        return self._category

    @property
    def category_short(self) -> Optional[str]:
        """The short name of the category for the current codepoint."""
        if self._unicode is None:
            return None

        if self._categoryShort is None:
            self._categoryShort = uniCat.get(self._unicode, "<undefined>")
        return self._categoryShort

    @property
    def char(self) -> Optional[str]:
        """The character for the current codepoint."""
        if self.unicode is None:
            return None

        return getUnicodeChar(self.unicode)

    @property
    def glyphname(self) -> Optional[str]:
        """The AGLFN glyph name for the current codepoint."""
        from jkUnicode.aglfn import getGlyphnameForUnicode

        return getGlyphnameForUnicode(self.unicode)

    @property
    def name(self) -> Optional[str]:
        """The Unicode name for the current codepoint."""
        if self._unicode is None:
            return None

        if self._name is None:
            self._name = self.uniName.get(self._unicode, None)
            # TODO: Add nicer names based on original Unicode names?
            if self._name is None:
                if 0xE000 <= self._unicode < 0xF8FF:
                    self._name = "<Private Use>"
                elif 0xD800 <= self._unicode < 0xDB7F:
                    self._name = "<Non Private Use High Surrogate #%i>" % (
                        self._unicode - 0xD8000
                    )
                elif 0xDB80 <= self._unicode < 0xDBFF:
                    self._name = "<Private Use High Surrogate #%i>" % (
                        self._unicode - 0xDB80
                    )
                elif 0xDC00 <= self._unicode < 0xDFFF:
                    self._name = "<Low Surrogate #%i>" % (
                        self._unicode - 0xDC00
                    )
                else:
                    self._name = "<undefined>"
        return self._name

    @property
    def nice_name(self) -> Optional[str]:
        """A more human-readable Unicode name for the current codepoint."""
        if self.name is None:
            return None

        for transform_function in nice_name_rules:
            result = transform_function(self.name)
            if result:
                return result

        return self.name.capitalize()

    @property
    def decomposition_mapping(self) -> List[int]:
        """The decomposition mapping for the current codepoint."""
        if self._dc_mapping is None:
            if self._unicode is not None:
                try:
                    if self._unicode is not None:
                        self._dc_mapping = uniDecompositionMapping[
                            self._unicode
                        ]
                except KeyError:
                    self._dc_mapping = []
            else:
                self._dc_mapping = []
        return self._dc_mapping

    @property
    def lc_mapping(self) -> Optional[int]:
        """The lowercase mapping for the current codepoint."""
        if self._unicode is None:
            return None

        if self._lc_mapping is None:
            self._lc_mapping = uniLowerCaseMapping.get(self._unicode, None)
        return self._lc_mapping

    @property
    def uc_mapping(self) -> Optional[int]:
        """The uppercase mapping for the current codepoint."""
        if self._unicode is None:
            return None

        if self._uc_mapping is None:
            self._uc_mapping = uniUpperCaseMapping.get(self._unicode, None)
        return self._uc_mapping

    @property
    def script(self) -> Optional[str]:
        if self._script is None:
            if self._unicode is None:
                return None
            self._script = get_script(self._unicode)
        return self._script


if __name__ == "__main__":
    print("\n*** Test of jkUnicode.UniInfo ***")
    for u in [9912, 80, 0x1E40]:
        j = UniInfo(u)
        print("Repr.:")
        print(j)
        print("- " * 20)
        print("             Name:", j.name)
        print("       Glyph Name:", j.glyphname)
        print("         Category:", j.category)
        print("           Script:", j.script)
        print(
            "    Decomposition:",
            " ".join([hex(n) for n in j.decomposition_mapping]),
        )
        print("        Character:", j.char)
        lc = j.lc_mapping
        print("Lowercase Mapping:", lc)
        if lc is not None:
            j.unicode = lc
            print(j)
        print("-" * 40)



from __future__ import annotations

import codecs
import json
import os

from pathlib import Path
from typing import Any

base_path = Path(os.path.split(Path(__file__).resolve().parent)[0])
json_path = base_path / "json"


def json_to_file(
    path: Path, file_name: str, obj: Any, human_readable: bool = True
) -> None:
    with codecs.open(str(path / f"{file_name}.json"), "w", "utf-8") as f:
        if human_readable:
            indent = 4
        else:
            indent = None
        json.dump(obj, f, ensure_ascii=False, indent=indent, sort_keys=True)


def dict_from_file(path: Path, file_name: str) -> Any:
    with codecs.open(str(path / f"{file_name}.json"), "r", "utf-8") as f:
        return json.load(f)


def clean_json_dir(path: Path):
    # Clean up the directory which contains the separate json files to avoid
    # orphaned files
    for file_path in path.iterdir():
        name = str(file_path)
        if not name[0] == "." and name.lower().endswith(".json"):
            try:
                (path / name).unlink()
            except FileNotFoundError:
                print(
                    "WARNING: Could not remove file before regenerating it:",
                    os.path.join(path, name),
                )
